/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 1996, California Institute of Technology.  
// ALL RIGHTS RESERVED.   U.S. Government Sponsorship acknowledged. 
//
// Please read the full copyright notice in the file COPYRIGH 
// in this directory.
//
// Author: Todd Karakashian, NASA/Jet Propulsion Laboratory
//         Todd.K.Karakashian@jpl.nasa.gov
//
// $RCSfile: hdf_dds.cc,v $ - server CGI to send the DDS
// 
// $Log: hdf_dds.cc,v $
// Revision 1.7  1998/09/17 17:57:50  jimg
// Resolved conflicts from previous merge
//
// Revision 1.6  1998/04/03 18:34:28  jimg
// Fixes for vgroups and Sequences from Jake Hamby
//
// Revision 1.5  1997/12/30 23:50:35  jimg
// Changed from the old-style filter program to one based on the new DODSFilter
// class.
//
// Revision 1.4  1997/03/10 22:45:53  jimg
// Update for 2.12
//
// Revision 1.3  1996/09/24 22:33:58  todd
// Added copyright statement.
//
// Revision 1.2  1996/07/22  20:32:33  todd
// Changes for DODS DAP 2.06.
//
// Revision 1.1  1996/05/02  18:13:47  todd
// Initial revision
//
//////////////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include <strstream.h>

#include "DDS.h"
#include "cgi_util.h"
#include "DODSFilter.h"
#include "hcerr.h"
#include "dhdferr.h"

extern void read_dds(DDS& dds, const String& cachedir, const String& filename);

int 
main(int argc, char *argv[])
{
    DDS dds;
    DODSFilter df(argc, argv);

    if (!df.OK()) {
	df.print_usage();
	return 1;
    }

    if (df.version()) {
	df.send_version_info();
	return 0;
    }

#ifndef NO_EXCEPTIONS
    try {
#endif
      read_dds(dds, df.get_cache_dir(), df.get_dataset_name());
      
      if (!df.read_ancillary_dds(dds))
	return 1;
      
      if (!df.send_dds(dds, true))
	return 1;

#ifndef NO_EXCEPTIONS
    }
    catch (dhdferr &d) {
        ostrstream s;
	s << "hdf_dds: " << d << ends;
        ErrMsgT(s.str());
	Error e(unknown_error, d.errmsg());
	set_mime_text(cout, dods_error);
	e.print(cout);
	return 1;
    }
    catch (hcerr &h) {
        ostrstream s;
	s << "hdf_dds: " << h << ends;
        ErrMsgT(s.str());
	Error e(unknown_error, h.errmsg());
	set_mime_text(cout, dods_error);
	e.print(cout);
	return 1;
    }
    catch (...) {
        String s("hdf_dds: Unknown exception");
	ErrMsgT(s);
	Error e(unknown_error, s);
	set_mime_text(cout, dods_error);
	e.print(cout);
	return 1;
    }
#endif

    return 0;
}
